﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Crm4ScriptExtractor.Model;
using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Spreadsheet;
using DocumentFormat.OpenXml;

namespace Crm4ScriptExtractor.DataAccess
{
    /// <summary>
    /// Open XML の Spreadsheet ML を使用してスクリプトモデルを出力します。
    /// </summary>
    class SpreadsheetScriptWriter : IScriptDataWriter
    {
        #region コンストラクタ
        public SpreadsheetScriptWriter(string filename)
        {
            _filename = filename;
        }
        #endregion

        private string _filename;

        #region IScriptDataWriter Members

        public void WriteData(ScriptDataModel model)
        {
            ProcessWriteData(model);
        }

        #endregion

        private void ProcessWriteData(ScriptDataModel model)
        {
            using (SpreadsheetDocument spreadsheetDocument = SpreadsheetDocument.Create(_filename
                , DocumentFormat.OpenXml.SpreadsheetDocumentType.Workbook))
            {
                // ワークブックの初期設定
                InitWorkbook(spreadsheetDocument);

                FillSheetData(model, spreadsheetDocument.WorkbookPart.WorksheetParts.First());

                spreadsheetDocument.WorkbookPart.Workbook.Save();
                spreadsheetDocument.Close();
            }
        }
        private void InitWorkbook(SpreadsheetDocument spreadsheetDocument)
        {
            // WorkbookPart を追加する
            WorkbookPart workbookPart = spreadsheetDocument.AddWorkbookPart();
            workbookPart.Workbook = new Workbook();

            // WorkbookPart に WorksheetPart を追加する
            WorksheetPart worksheetPart = workbookPart.AddNewPart<WorksheetPart>();
            worksheetPart.Worksheet = new Worksheet();
            worksheetPart.Worksheet.Append(CreateColumns());
            worksheetPart.Worksheet.Append(new SheetData());
            worksheetPart.Worksheet.Append(new AutoFilter() { Reference = "A1:F1" });

            // Workbook に Sheets を追加
            Sheets sheets = spreadsheetDocument.WorkbookPart.Workbook.AppendChild<Sheets>(new Sheets());

            // Worksheet の追加
            Sheet sheet = new Sheet() { Id = spreadsheetDocument.WorkbookPart.GetIdOfPart(worksheetPart), SheetId = 1, Name = "スクリプト一覧" };
            sheets.AppendChild(sheet);

            // スタイルの設定
            WorkbookStylesPart workbookStylesPart = workbookPart.AddNewPart<WorkbookStylesPart>();
            InitStyleSheet(workbookStylesPart);
        }
        private Columns CreateColumns()
        {
            Columns columns = new Columns();
            Column column1 = new Column() { Min = (UInt32Value)1U, Max = (UInt32Value)1U, Width = 22.0D, CustomWidth = true };
            Column column2 = new Column() { Min = (UInt32Value)2U, Max = (UInt32Value)2U, Width = 19.0D, CustomWidth = true };
            Column column3 = new Column() { Min = (UInt32Value)3U, Max = (UInt32Value)3U, Width = 17.0D, CustomWidth = true };
            Column column4 = new Column() { Min = (UInt32Value)4U, Max = (UInt32Value)4U, Width = 16.0D, CustomWidth = true };
            Column column5 = new Column() { Min = (UInt32Value)5U, Max = (UInt32Value)5U, Width = 14.0D, CustomWidth = true };
            Column column6 = new Column() { Min = (UInt32Value)6U, Max = (UInt32Value)6U, Width = 77.0D, BestFit = true, CustomWidth = true };

            columns.Append(column1);
            columns.Append(column2);
            columns.Append(column3);
            columns.Append(column4);
            columns.Append(column5);
            columns.Append(column6);
            return columns;
        }
        private void FillSheetData(ScriptDataModel model, WorksheetPart worksheetPart)
        {
            uint rowIndex = 1;
            SheetData sheetData = worksheetPart.Worksheet.GetFirstChild<SheetData>();
            sheetData.Append(CreateHeaderRow(model.Header, rowIndex));

            foreach (RowDataModel rowDataModel in model.Rows)
            {
                rowIndex++;
                sheetData.Append(CreateDataRow(rowDataModel, rowIndex));
            }

        }
        private Row CreateHeaderRow(RowHeaderModel rowHeaderModel, uint rowIndex)
        {
            Row row = new Row() { RowIndex = (UInt32Value)rowIndex, Spans = new ListValue<StringValue>() { InnerText = "1:6" } };
            row.Append(CreateHeaderCell("A" + rowIndex, rowHeaderModel.EntityLogicalName));
            row.Append(CreateHeaderCell("B" + rowIndex, rowHeaderModel.EntityDisplayName));
            row.Append(CreateHeaderCell("C" + rowIndex, rowHeaderModel.AttributeLogicalName));
            row.Append(CreateHeaderCell("D" + rowIndex, rowHeaderModel.AttributeDisplayName));
            row.Append(CreateHeaderCell("E" + rowIndex, rowHeaderModel.Event));
            row.Append(CreateHeaderCell("F" + rowIndex, rowHeaderModel.Script));

            return row;
        }
        private Cell CreateHeaderCell(string columnName, string text)
        {
            Cell cell = new Cell() { CellReference = columnName, StyleIndex = (UInt32Value)1U };
            cell.CellValue = new CellValue(text);
            cell.DataType = new EnumValue<CellValues>(CellValues.String);

            return cell;
        }
        private Row CreateDataRow(RowDataModel rowDataModel, uint rowIndex)
        {
            Row row = new Row() { RowIndex = (UInt32Value)rowIndex, Spans = new ListValue<StringValue>() { InnerText = "1:6" } };
            row.Append(CreateDataCell("A" + rowIndex, rowDataModel.EntityLogicalName));
            row.Append(CreateDataCell("B" + rowIndex, rowDataModel.EntityDisplayName));
            row.Append(CreateDataCell("C" + rowIndex, rowDataModel.AttributeLogicalName));
            row.Append(CreateDataCell("D" + rowIndex, rowDataModel.AttributeDisplayName));
            row.Append(CreateDataCell("E" + rowIndex, rowDataModel.EventType.ToString()));
            row.Append(CreateDataCell("F" + rowIndex, rowDataModel.Script));

            return row;
        }
        private Cell CreateDataCell(string columnName, string text)
        {
            Cell cell = new Cell() { CellReference = columnName, StyleIndex = (UInt32Value)2U };
            cell.CellValue = new CellValue(text);
            cell.DataType = new EnumValue<CellValues>(CellValues.String);

            return cell;
        }

        #region スタイルの設定
        /// <summary>
        /// スタイルシートの作成
        /// </summary>
        /// <param name="part"></param>
        private void InitStyleSheet(WorkbookStylesPart part)
        {
            Stylesheet stylesheet1 = new Stylesheet();

            Fonts fonts1 = new Fonts() { Count = (UInt32Value)2U };

            Font font1 = new Font();
            FontSize fontSize1 = new FontSize() { Val = 11D };
            Color color1 = new Color() { Theme = (UInt32Value)1U };
            FontName fontName1 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering1 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme1 = new FontScheme() { Val = FontSchemeValues.Minor };

            font1.Append(fontSize1);
            font1.Append(color1);
            font1.Append(fontName1);
            font1.Append(fontFamilyNumbering1);
            font1.Append(fontScheme1);

            Font font2 = new Font();
            FontSize fontSize2 = new FontSize() { Val = 9D };
            Color color2 = new Color() { Theme = (UInt32Value)1U };
            FontName fontName2 = new FontName() { Val = "Calibri" };
            FontFamilyNumbering fontFamilyNumbering2 = new FontFamilyNumbering() { Val = 2 };
            FontScheme fontScheme2 = new FontScheme() { Val = FontSchemeValues.Minor };

            font2.Append(fontSize2);
            font2.Append(color2);
            font2.Append(fontName2);
            font2.Append(fontFamilyNumbering2);
            font2.Append(fontScheme2);

            fonts1.Append(font1);
            fonts1.Append(font2);

            Fills fills1 = new Fills() { Count = (UInt32Value)3U };

            Fill fill1 = new Fill();
            PatternFill patternFill1 = new PatternFill() { PatternType = PatternValues.None };

            fill1.Append(patternFill1);

            Fill fill2 = new Fill();
            PatternFill patternFill2 = new PatternFill() { PatternType = PatternValues.Gray125 };

            fill2.Append(patternFill2);

            Fill fill3 = new Fill();

            PatternFill patternFill3 = new PatternFill() { PatternType = PatternValues.Solid };
            ForegroundColor foregroundColor1 = new ForegroundColor() { Rgb = "FFFFFF00" };
            BackgroundColor backgroundColor1 = new BackgroundColor() { Indexed = (UInt32Value)64U };

            patternFill3.Append(foregroundColor1);
            patternFill3.Append(backgroundColor1);

            fill3.Append(patternFill3);


            fills1.Append(fill1);
            fills1.Append(fill2);
            fills1.Append(fill3);

            Borders borders1 = new Borders() { Count = (UInt32Value)2U };

            Border border1 = new Border();
            LeftBorder leftBorder1 = new LeftBorder();
            RightBorder rightBorder1 = new RightBorder();
            TopBorder topBorder1 = new TopBorder();
            BottomBorder bottomBorder1 = new BottomBorder();
            DiagonalBorder diagonalBorder1 = new DiagonalBorder();

            border1.Append(leftBorder1);
            border1.Append(rightBorder1);
            border1.Append(topBorder1);
            border1.Append(bottomBorder1);
            border1.Append(diagonalBorder1);

            Border border2 = new Border();

            LeftBorder leftBorder2 = new LeftBorder() { Style = BorderStyleValues.Thin };
            Color color3 = new Color() { Indexed = (UInt32Value)64U };

            leftBorder2.Append(color3);

            RightBorder rightBorder2 = new RightBorder() { Style = BorderStyleValues.Thin };
            Color color4 = new Color() { Indexed = (UInt32Value)64U };

            rightBorder2.Append(color4);

            TopBorder topBorder2 = new TopBorder() { Style = BorderStyleValues.Thin };
            Color color5 = new Color() { Indexed = (UInt32Value)64U };

            topBorder2.Append(color5);

            BottomBorder bottomBorder2 = new BottomBorder() { Style = BorderStyleValues.Thin };
            Color color6 = new Color() { Indexed = (UInt32Value)64U };

            bottomBorder2.Append(color6);
            DiagonalBorder diagonalBorder2 = new DiagonalBorder();

            border2.Append(leftBorder2);
            border2.Append(rightBorder2);
            border2.Append(topBorder2);
            border2.Append(bottomBorder2);
            border2.Append(diagonalBorder2);

            borders1.Append(border1);
            borders1.Append(border2);

            CellStyleFormats cellStyleFormats1 = new CellStyleFormats() { Count = (UInt32Value)1U };
            CellFormat cellFormat1 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)0U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)0U };

            cellStyleFormats1.Append(cellFormat1);

            CellFormats cellFormats1 = new CellFormats() { Count = (UInt32Value)3U };
            CellFormat cellFormat2 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)0U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)0U, FormatId = (UInt32Value)0U };
            CellFormat cellFormat3 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)2U, BorderId = (UInt32Value)1U, FormatId = (UInt32Value)0U, ApplyFont = true, ApplyFill = true, ApplyBorder = true };

            CellFormat cellFormat4 = new CellFormat() { NumberFormatId = (UInt32Value)0U, FontId = (UInt32Value)1U, FillId = (UInt32Value)0U, BorderId = (UInt32Value)1U, FormatId = (UInt32Value)0U, ApplyFont = true, ApplyBorder = true, ApplyAlignment = true };
            Alignment alignment1 = new Alignment() { Vertical = VerticalAlignmentValues.Center, WrapText = true };

            cellFormat4.Append(alignment1);

            cellFormats1.Append(cellFormat2);
            cellFormats1.Append(cellFormat3);
            cellFormats1.Append(cellFormat4);

            CellStyles cellStyles1 = new CellStyles() { Count = (UInt32Value)1U };
            CellStyle cellStyle1 = new CellStyle() { Name = "Normal", FormatId = (UInt32Value)0U, BuiltinId = (UInt32Value)0U };

            cellStyles1.Append(cellStyle1);
            DifferentialFormats differentialFormats1 = new DifferentialFormats() { Count = (UInt32Value)0U };
            TableStyles tableStyles1 = new TableStyles() { Count = (UInt32Value)0U, DefaultTableStyle = "TableStyleMedium9", DefaultPivotStyle = "PivotStyleLight16" };

            Colors colors1 = new Colors();

            MruColors mruColors1 = new MruColors();
            Color color7 = new Color() { Rgb = "FF00FFFF" };

            mruColors1.Append(color7);

            colors1.Append(mruColors1);

            stylesheet1.Append(fonts1);
            stylesheet1.Append(fills1);
            stylesheet1.Append(borders1);
            stylesheet1.Append(cellStyleFormats1);
            stylesheet1.Append(cellFormats1);
            stylesheet1.Append(cellStyles1);
            stylesheet1.Append(differentialFormats1);
            stylesheet1.Append(tableStyles1);
            stylesheet1.Append(colors1);

            part.Stylesheet = stylesheet1;



        }
        #endregion
    }
}
